@@ -4,7 +4,7 @@
 #
 #  Created by Oliver Borchert on May 09, 2019.
 #  Copyright (c) 2019 Oliver Borchert. All rights reserved.
-#  
+#
 
 from abc import ABC
 
@@ -12,7 +12,6 @@ class CallbackException(Exception):
     """
     Exception raised by callbacks to stop the training procedure.
     """
-    pass
 
 
 class TrainingCallback(ABC):
@@ -25,8 +24,7 @@ class TrainingCallback(ABC):
     # MARK: Instance Methods
     def before_training(self, model, num_epochs):
         """
-        Method is called prior to the start of the training.
-        This method must not raise exceptions.
+        Method is called prior to the start of the training. This method must not raise exceptions.
 
         Parameters:
         -----------
@@ -35,13 +33,12 @@ class TrainingCallback(ABC):
         - num_epochs: int
             The maximum number of epochs performed during training.
         """
-        pass
 
     def before_epoch(self, current, num_iterations):
         """
-        Method is called at the begining of every epoch during training.
-        This method may raise exceptions if training should be stopped. Note,
-        however, that stopping training at this stage is an advanced scenario.
+        Method is called at the begining of every epoch during training. This method may raise
+        exceptions if training should be stopped. Note, however, that stopping training at this
+        stage is an advanced scenario.
 
         Parameters:
         -----------
@@ -50,46 +47,39 @@ class TrainingCallback(ABC):
         - num_iterations: int
             The expected number of iterations for the batch.
         """
-        pass
 
     def after_batch(self, train_loss):
         """
-        Method is called at the end of a mini-batch. If the data is not
-        partitioned into batches, this function is never called.
-        The method may not raise exceptions.
+        Method is called at the end of a mini-batch. If the data is not partitioned into batches,
+        this function is never called. The method may not raise exceptions.
 
         Parameters:
         -----------
         - train_loss: float
             The loss computed from the last mini-batch.
         """
-        pass
 
     def after_epoch(self, metrics):
         """
-        Method is called at the end of every epoch during training.
-        This method may raise exceptions if training should be stopped.
+        Method is called at the end of every epoch during training. This method may raise
+        exceptions if training should be stopped.
 
         Parameters:
         -----------
         - metrics: bxtorch.nn.training.wrappers.Evaluation
             Metrics obtained after training this epoch.
         """
-        pass
 
     def after_training(self):
         """
-        Method is called upon end of the training procedure.
-        The method may not raise exceptions.
+        Method is called upon end of the training procedure. The method may not raise exceptions.
         """
-        pass
 
 
 class PredictionCallback(ABC):
     """
-    Abstract class to be subclassed by all prediction callbacks.
-    These callbacks are passed to a trainer which calls the implemented methods
-    at specific points during inference.
+    Abstract class to be subclassed by all prediction callbacks. These callbacks are passed to a
+    trainer which calls the implemented methods at specific points during inference.
     """
 
     # MARK: Instance Methods
@@ -104,7 +94,6 @@ class PredictionCallback(ABC):
         - num_iterations: int
             The number of iterations/batches performed for prediction.
         """
-        pass
 
     def after_batch(self, *args):
         """
@@ -113,13 +102,11 @@ class PredictionCallback(ABC):
         Parameters:
         -----------
         - args: varargs
-            Usually empty, just to be able to implement both TrainingCallback
-            and PredictionCallback.
+            Usually empty, just to be able to implement both TrainingCallback and
+            PredictionCallback.
         """
-        pass
 
     def after_predictions(self):
         """
         Called after all predictions have been made.
         """
-        pass
