@@ -4,7 +4,7 @@
 #
 #  Created by Oliver Borchert on May 09, 2019.
 #  Copyright (c) 2019 Oliver Borchert. All rights reserved.
-#  
+#
 
 import copy
 from .base import TrainingCallback, CallbackException
@@ -24,14 +24,14 @@ class EarlyStopping(TrainingCallback):
         Parameters:
         -----------
         - metric: str or list of str, default: 'val_loss'
-            The metric to watch during training. If a list is given, the sum
-            of the given metrics is considered.
+            The metric to watch during training. If a list is given, the sum of the given metrics
+            is considered.
         - patience: int, default: 5
-            The number of epochs that training still continues although the
-            watched metric is greater than its smallest value during training.
+            The number of epochs that training still continues although the watched metric is
+            greater than its smallest value during training.
         - restore_best: bool, default: False
-            Whether the model's parameter should be set to the parameters which
-            showed the best performance in terms of the watched metric.
+            Whether the model's parameter should be set to the parameters which showed the best
+            performance in terms of the watched metric.
         - minimize: bool, default: True
             Whether to minimize or maximize the given metric.
         """
@@ -76,11 +76,9 @@ class EarlyStopping(TrainingCallback):
     def _is_metric_better(self, metrics):
         if self.minimize:
             return self._current_metric(metrics) < self.best_metric
-        else:
-            return self._current_metric(metrics) > self.best_metric
+        return self._current_metric(metrics) > self.best_metric
 
     def _current_metric(self, metrics):
         if isinstance(self.metric, list):
             return sum(metrics[m] for m in self.metric)
-        else:
-            return metrics[self.metric]
+        return metrics[self.metric]
