@@ -4,15 +4,15 @@
 #
 #  Created by Oliver Borchert on May 21, 2019.
 #  Copyright (c) 2019 Oliver Borchert. All rights reserved.
-#  
+#
 
 import os
 from .base import TrainingCallback, CallbackException
 
 class SynchronizationCallback(TrainingCallback):
     """
-    Synchronizer callback which can be used within a parallel environment to
-    stop and resume training at specific points.
+    Synchronizer callback which can be used within a parallel environment to stop and resume
+    training at specific points.
     """
 
     # MARK: Initialization
@@ -24,11 +24,11 @@ class SynchronizationCallback(TrainingCallback):
         -----------
         - push_queue: torch.multiprocessing.Queue
             The queue onto which the parallel environment pushes whether the
-            current epoch should be trained. If ``False`` is passed, training
+            current epoch should be trained. If `False` is passed, training
             is stopped.
         - pull_queue: torch.multiprocessing.Queue
-            The queue onto which the callback pushes ``True`` to indicate that 
-            the epoch is finished. Upon end of training, it pushes ``False``.
+            The queue onto which the callback pushes `True` to indicate that the epoch is finished.
+            Upon end of training, it pushes `False`.
         """
         self.push_queue = push_queue
         self.pull_queue = pull_queue
@@ -52,10 +52,10 @@ class SynchronizationCallback(TrainingCallback):
 
 class ModelSharingCallback(TrainingCallback):
     """
-    The model sharing callback should be used in a multiprocessing context
-    where a model is shared in memory, but workers are operating on a GPU and
-    cannot share the model directly. This callback ensures that a trainer
-    always reads updated from other workers at the beginning of every batch.
+    The model sharing callback should be used in a multiprocessing context where a model is shared
+    in memory, but workers are operating on a GPU and cannot share the model directly. This
+    callback ensures that a trainer always reads updated from other workers at the beginning of
+    every batch.
     """
 
     def __init__(self, shared_model):
@@ -65,8 +65,7 @@ class ModelSharingCallback(TrainingCallback):
         Parameters:
         -----------
         - shared_model: torch.nn.Module
-            The model that should be shared. Must have already been moved to 
-            shared memory.
+            The model that should be shared. Must have already been moved to shared memory.
         """
         self.shared_model = shared_model
         self.trained_model = None
