@@ -27,7 +27,14 @@ class NeptuneTracker(TrainingCallback):
         self.experiment = experiment
 
     def after_batch(self, train_loss):
-        self.experiment.log_metric('batch_train_loss', train_loss)
+        if isinstance(train_loss, (list, tuple)):
+            for i, val in enumerate(train_loss):
+                self.experiment.log_metric(f'batch_train_loss_{i}', val)
+        elif isinstance(train_loss, dict):
+            for k, val in train_loss.items():
+                self.experiment.log_metric(f'batch_train_loss_{k}', val)
+        else:
+            self.experiment.log_metric('batch_train_loss', train_loss)
 
     def after_epoch(self, metrics):
         for k, v in metrics.to_dict().items():
