@@ -4,9 +4,8 @@
 #
 #  Created by Oliver Borchert on May 20, 2019.
 #  Copyright (c) 2019 Oliver Borchert. All rights reserved.
-#  
+#
 
 from .gumbel import gumbel_softmax
-from .metrics import accuracy, precision, average_precision, recall, \
-    roc_auc_score
+from .metrics import accuracy, precision, average_precision, recall, roc_auc_score
 from .random import generate_noise
