@@ -8,14 +8,13 @@
 
 import torch
 import torch.jit as jit
-import torch.nn as nn
 import torch.nn.functional as F
 
 @jit.script
 def gumbel_softmax(logits: torch.Tensor,
-                   tau: float = 1, 
+                   tau: float = 1,
                    hard: bool = False,
-                   eps: float = 1e-10, 
+                   eps: float = 1e-10,
                    dim: int = -1):
     """
     Numerically stable version of PyTorch's builtin gumbel softmax.
@@ -42,5 +41,4 @@ def gumbel_softmax(logits: torch.Tensor,
         index = y_soft.max(dim, keepdim=True)[1]
         y_hard = torch.zeros_like(logits).scatter_(dim, index, 1.0)
         return y_hard - y_soft.detach() + y_soft
-    else:
-        return y_soft
+    return y_soft
