@@ -6,23 +6,23 @@
 #  Copyright (c) 2019 Oliver Borchert. All rights reserved.
 #
 
+from typing import List, Tuple, Optional
 import numpy as np
 import torch
 import torch.nn as nn
 import torch.jit as jit
-from typing import List, Tuple, Optional
 
 class StackedLSTM(nn.Module):
     """
-    The stacked LSTM is an extension to PyTorch's native LSTM allowing stacked
-    LSTMs with different hidden dimensions being stacked.
+    The stacked LSTM is an extension to PyTorch's native LSTM allowing stacked LSTMs with different
+    hidden dimensions being stacked. Furthermore, it allows using an LSTM on a GPU without cuDNN.
+    This is useful when higher-order gradients are required. In all other cases, it is best to use
+    PyTorch's builtin LSTM.
     """
 
     batch_first: jit.Final[bool]
 
-    # MARK: Initialization
-    def __init__(self, input_size, hidden_sizes, bias=True, batch_first=False,
-                 cudnn=True):
+    def __init__(self, input_size, hidden_sizes, bias=True, batch_first=False, cudnn=True):
         """
         Initializes a new stacked LSTM according to the given parameters.
 
@@ -38,52 +38,42 @@ class StackedLSTM(nn.Module):
             Whether the batch or the sequence can be found in the first
             dimension.
         - cudnn: bool, default: True
-            Whether to use PyTorch's LSTM implementation which uses cuDNN on
-            Nvidia GPUs. You usually don't want to change the default value,
-            however, PyTorch's default implementation does not allow higher-
-            order gradients of the LSTMCell as of version 1.1.0. When this
-            value is set to False, we therefore use a (slower) implementation
+            Whether to use PyTorch's LSTM implementation which uses cuDNN on Nvidia GPUs. You
+            usually don't want to change the default value, however, PyTorch's default
+            implementation does not allow higher-order gradients of the LSTMCell as of version
+            1.1.0. When this value is set to False, we therefore use a (slower) implementation
             of a LSTM cell which allows higher-order gradients.
         """
         super().__init__()
         self.batch_first = batch_first
-        self.stacked_cell = StackedLSTMCell(
-            input_size, hidden_sizes, bias=bias, cudnn=cudnn
-        )
+        self.stacked_cell = StackedLSTMCell(input_size, hidden_sizes, bias=bias, cudnn=cudnn)
 
     # MARK: Instance Methods
-    def forward(self, 
-                inputs: torch.Tensor,
-                initial_states: Optional[
-                    List[Tuple[torch.Tensor, torch.Tensor]]] = None, 
+    def forward(self, inputs: torch.Tensor,
+                initial_states: Optional[List[Tuple[torch.Tensor, torch.Tensor]]] = None,
                 return_sequence: bool = True):
         """
         Computes the forward pass through the stacked LSTM.
 
         Parameters:
         -----------
-        - inputs: torch.FloatTensor [S, B, N]
-            The inputs fed to the LSTM one after the other. Sequence length S,
-            batch size B, and input size N. If ``batch_first`` is set to True,
-            the first and second dimension should be swapped.
-        - initial_states: list of tuple of 
-                (torch.FloatTensor [H_i], torch.FloatTensor [H_i]),
-                default: None
-            The initial states for all LSTM layers. The length of the list must
-            match the number of layers in the LSTM, the sizes of the states must
-            match the hidden sizes of the LSTM layers. If None is given, the
-            initial states are defaulted to all zeros.
+        - inputs: torch.Tensor [S, B, N]
+            The inputs fed to the LSTM one after the other. Sequence length S, batch size B, and
+            input size N. If `batch_first` is set to True, the first and second dimension should
+            be swapped.
+        - initial_states: list of tuple of (torch.Tensor [H_i], torch.Tensor [H_i]), default: None
+            The initial states for all LSTM layers. The length of the list must match the number of
+            layers in the LSTM, the sizes of the states must match the hidden sizes of the LSTM
+            layers. If None is given, the initial states are defaulted to all zeros.
         - return_sequence: bool, default: True
-            Whether to return all outputs from the last LSTM layer or only the
-            last one.
+            Whether to return all outputs from the last LSTM layer or only the last one.
 
         Returns:
         --------
-        - torch.FloatTensor [S, B, K] or torch.FloatTensor [B, K]
-            Depending on whether sequences are returned, either all outputs
-            or only the output from the last cell are returned. If the stacked
-            LSTM was initialized with ``batch_first``, the first and second
-            dimension are swapped when sequences are returned.
+        - torch.Tensor [S, B, K] or torch.Tensor [B, K]
+            Depending on whether sequences are returned, either all outputs or only the output from
+            the last cell are returned. If the stacked LSTM was initialized with `batch_first`,
+            the first and second dimension are swapped when sequences are returned.
         """
         if self.batch_first:
             inputs = inputs.transpose(1, 0)
@@ -108,20 +98,18 @@ class StackedLSTM(nn.Module):
 
         if return_sequence:
             result = torch.stack(outputs)
-            if self.batch_first: 
+            if self.batch_first:
                 # set batch first, sequence length second
                 result = result.transpose(1, 0)
             return result
-        else:
-            return outputs[0]
+        return outputs[0]
 
 
 class StackedLSTMCell(nn.Module):
     """
-    Actually, the StackedLSTMCell can easily be constructed from existing 
-    modules, however, a bug in PyTorch's JIT compiler prevents implementing
-    anything where a stacked LSTM is used within a loop (see the following 
-    issue: https://github.com/pytorch/pytorch/issues/18143). Hence, this
+    Actually, the StackedLSTMCell can easily be constructed from existing modules, however, a bug
+    in PyTorch's JIT compiler prevents implementing anything where a stacked LSTM is used within a
+    loop (see the following issue: https://github.com/pytorch/pytorch/issues/18143). Hence, this
     class provides a single time step for a stacked LSTM.
     """
 
@@ -142,10 +130,9 @@ class StackedLSTMCell(nn.Module):
         - bias: bool, default: True
             Whether to use a bias term for the LSTM implementation.
         - cudnn: bool, default: True
-            Whether to not use cuDNN. In almost all cases, you don't want to
-            set this value to false, however, you will need to change it if
-            you want to compute higher-order derivatives of a network with
-            a stacked LSTM cell.
+            Whether to not use cuDNN. In almost all cases, you don't want to set this value to
+            false, however, you will need to change it if you want to compute higher-order
+            derivatives of a network with a stacked LSTM cell.
         """
         super().__init__()
 
@@ -155,15 +142,14 @@ class StackedLSTMCell(nn.Module):
 
         cells = []
         dims = zip([input_size] + hidden_sizes, hidden_sizes)
-        for i, (in_dim, out_dim) in enumerate(dims):
+        for in_dim, out_dim in dims:
             cells.append(cell_class(in_dim, out_dim, bias=bias))
         self.cells = nn.ModuleList(cells)
-        
+
     # MARK: Instance Methods
-    def forward(self,
-                x: torch.Tensor,
-                initial_states: Optional[
-                    List[Tuple[torch.Tensor, torch.Tensor]]] = None):
+    # pylint: disable=arguments-differ
+    def forward(self, x: torch.Tensor,
+                initial_states: Optional[List[Tuple[torch.Tensor, torch.Tensor]]] = None):
         """
         Computes the new hidden states and cell states for each stacked cell.
 
@@ -171,16 +157,15 @@ class StackedLSTMCell(nn.Module):
         -----------
         - x: torch.Tensor [B, N]
             The input with batch size B and dimension N.
-        - states: list of tuple of (torch.Tensor [B, D], torch.Tensor [B, D]),
-                default: None
-            The states for each of the cells where each state is expected to
-            have a size with batch size B and (respective) hidden dimension D.
+        - states: list of tuple of (torch.Tensor [B, D], torch.Tensor [B, D]), default: None
+            The states for each of the cells where each state is expected to have a size with batch
+            size B and (respective) hidden dimension D.
 
         Returns:
         --------
         - torch.Tensor [B, D]
-            The output, i.e. the hidden state of the deepest cell. Only given
-            for convenience as it can be extracted from the other return value.
+            The output, i.e. the hidden state of the deepest cell. Only given for convenience as it
+            can be extracted from the other return value.
         - list of tuple of (torch.Tensor [B, D], torch.Tensor [B, D])
             The new hidden states and cell states for all cells.
         """
@@ -192,7 +177,7 @@ class StackedLSTMCell(nn.Module):
             ]
         else:
             states = initial_states
-        
+
         i = 0
         for cell in self.cells:
             x, next_cell = cell(
@@ -206,10 +191,9 @@ class StackedLSTMCell(nn.Module):
 
 class _LSTMCell(nn.Module):
     """
-    LSTMCell which does not have cuDNN support but allows for higher-order
-    gradients.
-    Consult PyTorch's LSTMCell for documentation on the class's initialization
-    parameters and how to call it.
+    LSTMCell which does not have cuDNN support but allows for higher-order gradients.
+    Consult PyTorch's LSTMCell for documentation on the class's initialization parameters and how
+    to call it.
     """
 
     hidden_size: jit.Final[int]
@@ -238,13 +222,16 @@ class _LSTMCell(nn.Module):
 
     # MARK: Instance Methods
     def reset_parameters(self):
+        """
+        Resets the parameters of the model.
+        """
         sqrt_hidden = np.sqrt(1 / self.hidden_size)
         init_from = (-sqrt_hidden, sqrt_hidden)
         for p in self.parameters():
             nn.init.uniform_(p, *init_from)
 
-    def forward(self, 
-                x_in: torch.Tensor,
+    # pylint: disable=arguments-differ
+    def forward(self, x_in: torch.Tensor,
                 state: Optional[Tuple[torch.Tensor, torch.Tensor]] = None):
 
         if state is None:
