@@ -4,22 +4,20 @@
 #
 #  Created by Oliver Borchert on May 10, 2019.
 #  Copyright (c) 2019 Oliver Borchert. All rights reserved.
-#  
+#
 
 import copy
-from abc import ABC, abstractclassmethod
+from abc import ABC
 from collections import OrderedDict
 import json
 import torch
-import torch.nn as nn
 from bxtorch.utils.stdio import ensure_valid_directories
 
 class Config(ABC):
     """
-    Base class which standardizes configuration for PyTorch modules. For a
-    specific configuration subclass, the configuration values are automatically
-    deduced from the class's annotations. All annotations which do not define
-    a default value are considered required.
+    Base class which standardizes configuration for PyTorch modules. For a specific configuration
+    subclass, the configuration values are automatically deduced from the class's annotations. All
+    annotations which do not define a default value are considered required.
     """
 
     # MARK: Static Methods
@@ -31,8 +29,8 @@ class Config(ABC):
         Parameters:
         -----------
         - file: str
-            The full path to the file. The extension does not need to be
-            specified, but must be .json.
+            The full path to the file. The extension does not need to be specified, but must be
+            .json.
 
         Returns:
         --------
@@ -47,21 +45,22 @@ class Config(ABC):
     @classmethod
     def _get_fields(cls):
         """
-        Aggregates all available fields and their types from the class itself
-        and its superclass (assuming it is a configuration as well).
+        Aggregates all available fields and their types from the class itself and its superclass
+        (assuming it is a configuration as well).
 
         Returns:
         --------
         - dict
             Mapping from field names to types.
         """
+        # pylint: disable=no-member
         annotations = {}
         if hasattr(super(), '_get_fields'):
             annotations = {**annotations, **super()._get_fields()}
         if hasattr(cls, '__annotations__'):
             annotations = {**annotations, **cls.__annotations__}
         return annotations
-    
+
     # MARK: Initialization
     def __init__(self, **kwargs):
         """
@@ -96,7 +95,7 @@ class Config(ABC):
                     f'Parameter {k} should have type {all_fields[k]} but '
                     f'has type {type(v)}.'
                 )
-        
+
         self._params = params
 
         # 5) Finally, set parameters on self and validate
@@ -117,8 +116,8 @@ class Config(ABC):
         Parameters:
         -----------
         - file: str
-            The full path to the file. The extension does not need to be
-            specified, but must be .json.
+            The full path to the file. The extension does not need to be specified, but must be
+            .json.
         """
         ensure_valid_directories(file)
         if not file.endswith('.json'):
@@ -128,9 +127,9 @@ class Config(ABC):
 
     def is_valid(self):
         """
-        Checks whether the given configuration is valid. This method should be
-        overridden if required. The default implementation implies validity for
-        all inputs matching the specified types.
+        Checks whether the given configuration is valid. This method should be overridden if
+        required. The default implementation implies validity for all inputs matching the specified
+        types.
 
         Returns:
         --------
@@ -146,34 +145,33 @@ class Config(ABC):
 
 class Configurable(ABC):
     """
-    Mixin class to make torch.nn.Module configurable in an easy and standardized
-    way. For initialization, a module is then given a single config file of 
-    type ``bxtorch.nn.Config``. Properties of the configuration can be easily 
-    accessed via ``self.<property>``. Additionally, this class makes it easy
-    to save and load the module in a consistent way. The binding to the
-    configuration class is achieved by setting the __
+    Mixin class to make torch.nn.Module configurable in an easy and standardized way. For
+    initialization, a module is then given a single config file of type `bxtorch.nn.Config`.
+    Properties of the configuration can be easily accessed via `self.<property>`. Additionally,
+    this class makes it easy to save and load the module in a consistent way. The binding to the
+    configuration class is achieved by setting the __config__ class property to the class of the
+    configuration.
 
-    When subclassing a PyTorch module, make sure to include the Configurable
-    mixin as *first* dependency.
+    When subclassing a PyTorch module, make sure to include the Configurable mixin as *first*
+    dependency.
 
     Note:
     -----
-    Only top-level modules should use this mixin to profit from saving/loading
-    modules easily.
+    Only top-level modules should use this mixin to profit from saving/loading modules easily.
     """
 
     # MARK: Static Methods
     @classmethod
     def load(cls, file):
         """
-        Loads the model by loading a configuration defining the architecture
-        and another file containing the weights for this architecture.
+        Loads the model by loading a configuration defining the architecture and another file
+        containing the weights for this architecture.
 
         Parameters:
         -----------
         - file: str
-            The full path to the file. The configuration will be loaded from
-            <file>.json and the model's weights will be loaded from <file>.pt.
+            The full path to the file. The configuration will be loaded from <file>.json and the
+            model's weights will be loaded from <file>.pt.
 
         Returns:
         --------
@@ -192,19 +190,18 @@ class Configurable(ABC):
     # MARK: Initialization
     def __init__(self, *args, **kwargs):
         """
-        Initializes the configurable model. You can either pass an instance of
-        the model's configuration class or the keyword arguments which you
-        would normally pass to initialize the configuration instance. The
-        configuration is then created implicitly.
+        Initializes the configurable model. You can either pass an instance of the model's
+        configuration class or the keyword arguments which you would normally pass to initialize
+        the configuration instance. The configuration is then created implicitly.
 
         Parameters:
         -----------
         - args: variadic arguments
-            Either zero or one parameters. If one is given, it must be an
-            instance of the model's configuration class.
+            Either zero or one parameters. If one is given, it must be an instance of the model's
+            configuration class.
         - kwargs: keyword arguments
-            The arguments passed to the initializer of the configuration class.
-            Must only be given if the configuration instance is not defined.
+            The arguments passed to the initializer of the configuration class. Must only be given
+            if the configuration instance is not defined.
         """
         super(Configurable, self).__init__()
 
@@ -218,7 +215,7 @@ class Configurable(ABC):
             assert args[0] is None or \
                 isinstance(args[0], type(self).__config__), \
                 "Given configuration does not match the model's config class."
-            
+
             self._config = args[0]
         else:
             self._config = type(self).__config__(**kwargs)
@@ -256,9 +253,10 @@ class Configurable(ABC):
         Parameters:
         -----------
         - file: str
-            The full path to the file. The configuration will be saved to
-            <file>.json while the model's weights will be saved to <file>.pt.
+            The full path to the file. The configuration will be saved to <file>.json while the
+            model's weights will be saved to <file>.pt.
         """
+        # pylint: disable=protected-access
         ensure_valid_directories(file)
         self._config.save(file)
         # Ensure saving CPU model
