@@ -4,16 +4,15 @@
 #
 #  Created by Oliver Borchert on May 10, 2019.
 #  Copyright (c) 2019 Oliver Borchert. All rights reserved.
-#  
+#
 
 import torch.nn as nn
 
 class DataParallel(nn.DataParallel):
     """
-    Replacement for PyTorch's default ``nn.DataParallel`` class. It enables
-    accessing the attributes of the underlying module. If the attribute being
-    accessed is a module itself, it is also returned being wrapped as
-    DataParallel. Call ``.module`` to access the raw model.
+    Replacement for PyTorch's default `nn.DataParallel` class. It enables accessing the
+    attributes of the underlying module. If the attribute being accessed is a module itself, it is
+    also returned being wrapped as DataParallel. Call `.module` to access the raw model.
     """
 
     # MARK: Initialization
